@@ -23,6 +23,12 @@ class EventHandler:
             "CHANNEL_UPDATE": ChannelUpdate,
             "CHANNEL_DELETE": ChannelDelete,
             "CHANNEL_PINS_UPDATE": ChannelPinsUpdate,
+            "THREAD_CREATE": ThreadCreate,
+            "THREAD_UPDATE": ThreadUpdate,
+            "THREAD_DELETE": ThreadDelete,
+            "THREAD_LIST_SYNC": ThreadListSync,
+            "THREAD_MEMBER_UPDATE": ThreadMemberUpdate,
+            "THREAD_MEMBERS_UPDATE": ThreadMembersUpdate,
             "GUILD_CREATE": GuildCreate,
             "GUILD_UPDATE": GuildUpdate,
             "GUILD_DELETE": GuildDelete,
