@@ -2,7 +2,7 @@ import typing
 from .slashcommands import ApplicationCommandInteractionDataResolved
 from .components import Component, ComponentTypes
 from ..channel import Embed, AllowedMentions
-from ..guild import Member
+from ..guild import GuildMember
 from ..snowflake import Snowflake
 from ..user import User
 from ...base.model import TypeBase, FlagBase
@@ -14,11 +14,11 @@ class Interaction:
         self.id = Snowflake(resp["id"])
         self.application_id = Snowflake(resp["application_id"])
         self.type = InteractionType(resp["type"])
-        self.data = ApplicationCommandInteractionData(client, resp.get("data")) if self.type.application_command else resp.get("data")
+        self.data = ApplicationCommandInteractionData(client, resp.get("data")) if "data" in resp else resp.get("data")
         self.guild_id = Snowflake(resp.get("guild_id"))
         self.channel_id = Snowflake(resp.get("channel_id"))
         self.__member = resp.get("member")
-        self.member = Member.create(client, self.__member, guild_id=self.guild_id) if self.__member else None
+        self.member = GuildMember.create(client, self.__member, guild_id=self.guild_id) if self.__member else None
         self.__user = resp.get("user")
         self.user = User.create(client, self.__user) if self.__user else None
         self.token = resp["token"]
@@ -46,13 +46,14 @@ class InteractionType(TypeBase):
 
 class ApplicationCommandInteractionData:
     def __init__(self, client, resp: dict):
-        self.id = Snowflake(resp["id"])
-        self.name = resp["name"]
+        self.id = Snowflake.optional(resp.get("id"))
+        self.name = resp.get("name")
         self.__resolved = resp.get("resolved")
         self.resolved = ApplicationCommandInteractionDataResolved(client, resp.get("resolved")) if self.__resolved else self.__resolved
         self.options = resp.get("options")
         self.custom_id = resp.get("custom_id")
         self.component_type = ComponentTypes(resp.get("component_type")) if resp.get("component_type") else None
+        self.values = resp.get("values", [])
 
 
 class InteractionCallbackType(TypeBase):
@@ -63,9 +64,8 @@ class InteractionCallbackType(TypeBase):
     UPDATE_MESSAGE = 7
 
 
-class InteractionCallbackFlags(FlagBase):
-    # This actually still doesn't exist.
-    EPHEMERAL = 64
+class InteractionApplicationCommandCallbackDataFlags(FlagBase):
+    EPHEMERAL = 1 << 6
 
 
 class InteractionApplicationCommandCallbackData:
@@ -74,13 +74,13 @@ class InteractionApplicationCommandCallbackData:
                  content: str = None,
                  embeds: typing.List[Embed] = None,
                  allowed_mentions: AllowedMentions = None,
-                 flags: int = None,
+                 flags: typing.Union[InteractionApplicationCommandCallbackDataFlags, int] = None,
                  components: typing.List[typing.Union[dict, Component]] = None):
         self.tts = tts
         self.content = content
         self.embeds = embeds
         self.allowed_mentions = allowed_mentions
-        self.flags = flags
+        self.flags = InteractionApplicationCommandCallbackDataFlags.from_value(flags) if isinstance(flags, int) else flags
         self.components = components
 
     def to_dict(self):
